//
//    Copyright 2005, Thomas C. McDermott, N5EG
//    This file is part of SIGGEN - the Signal Generator program.
//
//    SIGGEN is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    SIGGEN is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SIGGEN, if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
#pragma once

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::IO;

#include "FrequencySelector.h"
#include "DataDisplay.h"
#include "USB_EZ_interface.h"

namespace SGENR1
{
	/// <summary> 
	/// Summary for ReferenceSelection
	///
	/// WARNING: If you change the name of this class, you will need to change the 
	///          'Resource File Name' property for the managed resource compiler tool 
	///          associated with all .resx files this class depends on.  Otherwise,
	///          the designers will not be able to interact properly with localized
	///          resources associated with this form.
	/// </summary>
	public __gc class ReferenceSelection : public System::Windows::Forms::Form
	{
	public: ReferenceSelection(VNADevice *vna, FrequencySelector *FreqSel, String *StartUpPath)
	{
		InitializeComponent();
		Fsel = FreqSel;
		StartUpDir = StartUpPath;
		VNADev = vna;
		InternalRefFreqError = 0;
		updateIntRefFreqError = false;		// don't update internal ref freq error

		// Update panel display to match retrieved reference parameters

		if (FreqSel->internalRef)
		{
			Internalpanel->Enabled = true;
			Externalpanel->Enabled = false;
			InternalReference->Checked = true;
			ExternalReference->Checked = false;
		}
		else
		{
			Internalpanel->Enabled = false;
			Externalpanel->Enabled = true;
			InternalReference->Checked = false;
			ExternalReference->Checked = true;
		}
		RefFreqtextBox->Text = FreqSel->refFreq.ToString("F6");

	}
        
	protected: 
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}

	private: FrequencySelector* Fsel;
	private: String* StartUpDir;
	private: VNADevice * VNADev;
	private: int InternalRefFreqError;
	private: bool updateIntRefFreqError;

	private: System::Windows::Forms::RadioButton *  InternalReference;
	private: System::Windows::Forms::RadioButton *  ExternalReference;

	private: System::Windows::Forms::Label *  label1;
	private: System::Windows::Forms::Label *  label2;
	private: System::Windows::Forms::Label *  label3;


	private: System::Windows::Forms::Button *  OKbutton;
	private: System::Windows::Forms::Button *  Cancelbutton;
	private: System::Windows::Forms::Label *  label5;
	private: System::Windows::Forms::Label *  label6;
	private: System::Windows::Forms::Label *  label7;
	private: System::Windows::Forms::Label *  label8;
	private: System::Windows::Forms::Panel *  Externalpanel;
	private: System::Windows::Forms::Panel *  Internalpanel;


	private: System::Windows::Forms::TextBox *  RefFreqtextBox;
	private: System::Windows::Forms::Label *  label4;
	private: System::Windows::Forms::GroupBox*  groupBox1;
	private: System::Windows::Forms::TextBox*  InternalCalFreqText;



	private: System::Windows::Forms::Button*  RecordCapturedFreq;

	private: System::Windows::Forms::Button*  StartInternalCal;

	private: System::Windows::Forms::Label*  label9;
	private: System::Windows::Forms::Label*  label10;
	private: System::Windows::Forms::Label*  label11;


	private:
		/// <summary>
		/// Required designer variable.
		/// </summary>
		System::ComponentModel::Container* components;

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
			this->InternalReference = (new System::Windows::Forms::RadioButton());
			this->ExternalReference = (new System::Windows::Forms::RadioButton());
			this->RefFreqtextBox = (new System::Windows::Forms::TextBox());
			this->label1 = (new System::Windows::Forms::Label());
			this->label2 = (new System::Windows::Forms::Label());
			this->label3 = (new System::Windows::Forms::Label());
			this->OKbutton = (new System::Windows::Forms::Button());
			this->Cancelbutton = (new System::Windows::Forms::Button());
			this->label5 = (new System::Windows::Forms::Label());
			this->label6 = (new System::Windows::Forms::Label());
			this->label7 = (new System::Windows::Forms::Label());
			this->label8 = (new System::Windows::Forms::Label());
			this->Externalpanel = (new System::Windows::Forms::Panel());
			this->label4 = (new System::Windows::Forms::Label());
			this->Internalpanel = (new System::Windows::Forms::Panel());
			this->groupBox1 = (new System::Windows::Forms::GroupBox());
			this->label11 = (new System::Windows::Forms::Label());
			this->label10 = (new System::Windows::Forms::Label());
			this->InternalCalFreqText = (new System::Windows::Forms::TextBox());
			this->RecordCapturedFreq = (new System::Windows::Forms::Button());
			this->StartInternalCal = (new System::Windows::Forms::Button());
			this->label9 = (new System::Windows::Forms::Label());
			this->Externalpanel->SuspendLayout();
			this->Internalpanel->SuspendLayout();
			this->groupBox1->SuspendLayout();
			this->SuspendLayout();
			// 
			// InternalReference
			// 
			this->InternalReference->Checked = true;
			this->InternalReference->Font = (new System::Drawing::Font(S"Microsoft Sans Serif", 9.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				(System::Byte)0));
			this->InternalReference->Location = System::Drawing::Point(48, 39);
			this->InternalReference->Name = S"InternalReference";
			this->InternalReference->Size = System::Drawing::Size(152, 24);
			this->InternalReference->TabIndex = 1;
			this->InternalReference->TabStop = true;
			this->InternalReference->Text = S"Internal Reference";
			this->InternalReference->CheckedChanged += new System::EventHandler(this, &ReferenceSelection::InternalReference_CheckedChanged);
			// 
			// ExternalReference
			// 
			this->ExternalReference->Font = (new System::Drawing::Font(S"Microsoft Sans Serif", 9.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				(System::Byte)0));
			this->ExternalReference->Location = System::Drawing::Point(48, 144);
			this->ExternalReference->Name = S"ExternalReference";
			this->ExternalReference->Size = System::Drawing::Size(168, 24);
			this->ExternalReference->TabIndex = 2;
			this->ExternalReference->Text = S"External Reference";
			// 
			// RefFreqtextBox
			// 
			this->RefFreqtextBox->AcceptsReturn = true;
			this->RefFreqtextBox->ForeColor = System::Drawing::SystemColors::ControlText;
			this->RefFreqtextBox->Location = System::Drawing::Point(232, 64);
			this->RefFreqtextBox->Name = S"RefFreqtextBox";
			this->RefFreqtextBox->Size = System::Drawing::Size(232, 20);
			this->RefFreqtextBox->TabIndex = 0;
			this->RefFreqtextBox->Leave += new System::EventHandler(this, &ReferenceSelection::RefFreqLeave);
			// 
			// label1
			// 
			this->label1->Location = System::Drawing::Point(48, 8);
			this->label1->Name = S"label1";
			this->label1->Size = System::Drawing::Size(376, 24);
			this->label1->TabIndex = 3;
			this->label1->Text = S"Make sure that no external reference is connected to the external jumper.";
			// 
			// label2
			// 
			this->label2->Location = System::Drawing::Point(48, 56);
			this->label2->Name = S"label2";
			this->label2->Size = System::Drawing::Size(400, 16);
			this->label2->TabIndex = 4;
			this->label2->Text = S"Reference Frequency: Approximately 24.000 MHz.   Reference Multiplier: 12x";
			// 
			// label3
			// 
			this->label3->ForeColor = System::Drawing::SystemColors::ControlText;
			this->label3->Location = System::Drawing::Point(48, 64);
			this->label3->Name = S"label3";
			this->label3->Size = System::Drawing::Size(176, 32);
			this->label3->TabIndex = 5;
			this->label3->Text = S"Reference Frequency, Hz. (Must be within range 5 to 75 MHz";
			// 
			// OKbutton
			// 
			this->OKbutton->DialogResult = System::Windows::Forms::DialogResult::OK;
			this->OKbutton->FlatStyle = System::Windows::Forms::FlatStyle::System;
			this->OKbutton->Location = System::Drawing::Point(160, 461);
			this->OKbutton->Name = S"OKbutton";
			this->OKbutton->Size = System::Drawing::Size(90, 24);
			this->OKbutton->TabIndex = 8;
			this->OKbutton->Text = S"OK";
			this->OKbutton->Click += new System::EventHandler(this, &ReferenceSelection::OKbutton_Click);
			// 
			// Cancelbutton
			// 
			this->Cancelbutton->DialogResult = System::Windows::Forms::DialogResult::Cancel;
			this->Cancelbutton->FlatStyle = System::Windows::Forms::FlatStyle::System;
			this->Cancelbutton->Location = System::Drawing::Point(293, 461);
			this->Cancelbutton->Name = S"Cancelbutton";
			this->Cancelbutton->Size = System::Drawing::Size(90, 24);
			this->Cancelbutton->TabIndex = 9;
			this->Cancelbutton->Text = S"Cancel";
			// 
			// label5
			// 
			this->label5->ForeColor = System::Drawing::SystemColors::ControlText;
			this->label5->Location = System::Drawing::Point(48, 8);
			this->label5->Name = S"label5";
			this->label5->Size = System::Drawing::Size(328, 23);
			this->label5->TabIndex = 10;
			this->label5->Text = S"Make sure that internal reference jumper is removed.";
			// 
			// label6
			// 
			this->label6->ForeColor = System::Drawing::SystemColors::ControlText;
			this->label6->Location = System::Drawing::Point(48, 32);
			this->label6->Name = S"label6";
			this->label6->Size = System::Drawing::Size(400, 23);
			this->label6->TabIndex = 11;
			this->label6->Text = S"Apply AC-coupled Reference Clock to External Reference header && ground.";
			// 
			// label7
			// 
			this->label7->Font = (new System::Drawing::Font(S"Microsoft Sans Serif", 14.25F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				(System::Byte)0));
			this->label7->Location = System::Drawing::Point(24, 8);
			this->label7->Name = S"label7";
			this->label7->Size = System::Drawing::Size(495, 23);
			this->label7->TabIndex = 12;
			this->label7->Text = S"Reference Selection / Calibration";
			this->label7->TextAlign = System::Drawing::ContentAlignment::MiddleCenter;
			// 
			// label8
			// 
			this->label8->Location = System::Drawing::Point(48, 32);
			this->label8->Name = S"label8";
			this->label8->Size = System::Drawing::Size(344, 23);
			this->label8->TabIndex = 13;
			this->label8->Text = S"Connect a shorting strap between the internal reference pins.";
			// 
			// Externalpanel
			// 
			this->Externalpanel->Controls->Add(this->label4);
			this->Externalpanel->Controls->Add(this->label5);
			this->Externalpanel->Controls->Add(this->label6);
			this->Externalpanel->Controls->Add(this->label3);
			this->Externalpanel->Controls->Add(this->RefFreqtextBox);
			this->Externalpanel->Enabled = false;
			this->Externalpanel->Location = System::Drawing::Point(12, 168);
			this->Externalpanel->Name = S"Externalpanel";
			this->Externalpanel->Size = System::Drawing::Size(518, 144);
			this->Externalpanel->TabIndex = 14;
			// 
			// label4
			// 
			this->label4->Location = System::Drawing::Point(48, 104);
			this->label4->Name = S"label4";
			this->label4->Size = System::Drawing::Size(408, 32);
			this->label4->TabIndex = 12;
			this->label4->Text = S"The highest allowable Ref Multipler value (given the reference frequency) will be" 
				S" automatically selected.";
			// 
			// Internalpanel
			// 
			this->Internalpanel->Controls->Add(this->label1);
			this->Internalpanel->Controls->Add(this->label8);
			this->Internalpanel->Controls->Add(this->label2);
			this->Internalpanel->Location = System::Drawing::Point(12, 63);
			this->Internalpanel->Name = S"Internalpanel";
			this->Internalpanel->Size = System::Drawing::Size(518, 80);
			this->Internalpanel->TabIndex = 15;
			// 
			// groupBox1
			// 
			this->groupBox1->Controls->Add(this->label11);
			this->groupBox1->Controls->Add(this->label10);
			this->groupBox1->Controls->Add(this->InternalCalFreqText);
			this->groupBox1->Controls->Add(this->RecordCapturedFreq);
			this->groupBox1->Controls->Add(this->StartInternalCal);
			this->groupBox1->Controls->Add(this->label9);
			this->groupBox1->Location = System::Drawing::Point(12, 321);
			this->groupBox1->Name = S"groupBox1";
			this->groupBox1->Size = System::Drawing::Size(518, 135);
			this->groupBox1->TabIndex = 16;
			this->groupBox1->TabStop = false;
			this->groupBox1->Text = S"Optional";
			// 
			// label11
			// 
			this->label11->Location = System::Drawing::Point(21, 95);
			this->label11->Name = S"label11";
			this->label11->Size = System::Drawing::Size(116, 35);
			this->label11->TabIndex = 5;
			this->label11->Text = S"Record Frequency to nearest 1 hertz.";
			// 
			// label10
			// 
			this->label10->Location = System::Drawing::Point(20, 43);
			this->label10->Name = S"label10";
			this->label10->Size = System::Drawing::Size(284, 47);
			this->label10->TabIndex = 4;
			this->label10->Text = S"Connect Frequency counter to Transmit connector.  Click Start Calibration button." 
				S" Enter frequency to nearest hertz, then click Enter";
			// 
			// InternalCalFreqText
			// 
			this->InternalCalFreqText->Location = System::Drawing::Point(143, 98);
			this->InternalCalFreqText->Name = S"InternalCalFreqText";
			this->InternalCalFreqText->Size = System::Drawing::Size(161, 20);
			this->InternalCalFreqText->TabIndex = 3;
			// 
			// RecordCapturedFreq
			// 
			this->RecordCapturedFreq->FlatStyle = System::Windows::Forms::FlatStyle::System;
			this->RecordCapturedFreq->Location = System::Drawing::Point(323, 95);
			this->RecordCapturedFreq->Name = S"RecordCapturedFreq";
			this->RecordCapturedFreq->Size = System::Drawing::Size(137, 27);
			this->RecordCapturedFreq->TabIndex = 2;
			this->RecordCapturedFreq->Text = S"Enter";
			this->RecordCapturedFreq->UseVisualStyleBackColor = true;
			this->RecordCapturedFreq->Click += new System::EventHandler(this, &ReferenceSelection::RecordCapturedFreq_Click);
			// 
			// StartInternalCal
			// 
			this->StartInternalCal->FlatStyle = System::Windows::Forms::FlatStyle::System;
			this->StartInternalCal->Location = System::Drawing::Point(323, 48);
			this->StartInternalCal->Name = S"StartInternalCal";
			this->StartInternalCal->Size = System::Drawing::Size(140, 29);
			this->StartInternalCal->TabIndex = 1;
			this->StartInternalCal->Text = S"Start 100 MHz. Calibration";
			this->StartInternalCal->UseVisualStyleBackColor = true;
			this->StartInternalCal->Click += new System::EventHandler(this, &ReferenceSelection::StartInternalCal_Click);
			// 
			// label9
			// 
			this->label9->Font = (new System::Drawing::Font(S"Microsoft Sans Serif", 9.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				(System::Byte)0));
			this->label9->Location = System::Drawing::Point(8, 12);
			this->label9->Name = S"label9";
			this->label9->Size = System::Drawing::Size(496, 27);
			this->label9->TabIndex = 0;
			this->label9->Text = S"Internal Oscillator Calibration (must have internal ref jumper connected)";
			this->label9->TextAlign = System::Drawing::ContentAlignment::MiddleCenter;
			// 
			// ReferenceSelection
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->BackColor = System::Drawing::SystemColors::ControlLight;
			this->ClientSize = System::Drawing::Size(552, 492);
			this->Controls->Add(this->groupBox1);
			this->Controls->Add(this->label7);
			this->Controls->Add(this->Cancelbutton);
			this->Controls->Add(this->OKbutton);
			this->Controls->Add(this->InternalReference);
			this->Controls->Add(this->ExternalReference);
			this->Controls->Add(this->Externalpanel);
			this->Controls->Add(this->Internalpanel);
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::Fixed3D;
			this->Name = S"ReferenceSelection";
			this->Text = S"ReferenceSelection";
			this->Externalpanel->ResumeLayout(false);
			this->Externalpanel->PerformLayout();
			this->Internalpanel->ResumeLayout(false);
			this->groupBox1->ResumeLayout(false);
			this->groupBox1->PerformLayout();
			this->ResumeLayout(false);

		}		

private: System::Void InternalReference_CheckedChanged(System::Object *  sender, System::EventArgs *  e)
		{
			if (InternalReference->Checked)
			{
				Internalpanel->Enabled = true;
				Externalpanel->Enabled = false;
			}
			else
			{
				Internalpanel->Enabled = false;
				Externalpanel->Enabled = true;
			}
		}


private: System::Void OKbutton_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 // User has updated the reference values. Load them back into the parent object.

			 if(InternalReference->Checked)
				 Fsel->internalRef = true ;	// Set internal reference mode
			 else							// User-selected values chosen (External mode)
			 {
				 Fsel->internalRef = false;
				 Fsel->refFreq = Convert::ToDouble(RefFreqtextBox->Text);
				 
				 // Now compute best value for reference multiplier

				int mult = (int)(300000000.0 / Fsel->refFreq);

				// Bound the range

				if(mult > 20)
					mult = 20;

				Fsel->refMult = mult;
			 }

			 if(updateIntRefFreqError)		// valid internal cal frequency error captured
				 Fsel->internalRefFreqError = InternalRefFreqError;

			 Fsel->Save(StartUpDir);		// Save the values to disk
		 }

 private: System::Void RefFreqLeave(System::Object * sender, System::EventArgs * e)
		  {
			  // test if the values are in the allowed range of 5 - 75 MHz.

			  double value = Convert::ToDouble(RefFreqtextBox->Text);
			  if(value < 5000000.0)
				  MessageBox::Show(S"Frequency must be at least 5000000.0 Hz", S"Error", 
				  MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
              
			  if(value > 75000000.0)
				  MessageBox::Show(S"Frequency must be no more than 75000000.0 Hz", S"Error",
					MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
				 
		  }



private: System::Void StartInternalCal_Click(System::Object*  sender, System::EventArgs*  e)
		 {
			 // Generate 100.000 MHz on the Transmit connector at 0 dbm level

			 Fsel->internalRef = true;	// Set FrequencySelector to Internal mode

			 SGEN_TXBUFFER* tbuf = new SGEN_TXBUFFER;
			 SGEN_RXBUFFER* rbuf = new SGEN_RXBUFFER;

			 tbuf->PLLmult = Fsel->refMult + 64;			// Makes sure PLL range bit is set high
			 tbuf->ReplyType = SGEN_REPLYTYPE_SETPLL;

			 VNADev->WriteRead(tbuf, rbuf);					// update PLL

			 tbuf->TxAccum = Fsel->FreqToDDS(100000000.0);	// no correction applied
			 tbuf->IDAClevel = TxLevLinear(0.0);
			 tbuf->ReplyType = SGEN_REPLYTYPE_SETFREQ;

			 VNADev->WriteRead(tbuf, rbuf);					// set frequency
		 }

private: System::Void RecordCapturedFreq_Click(System::Object*  sender, System::EventArgs*  e)
		 {
			 // Capture recorded frequency. Check that it's within reasonable limits.

			 int CaptFreq;

			 try
			 {
				 String *input = InternalCalFreqText->Text;
				 CaptFreq = Convert::ToInt32(input);
			 }
			 catch (Exception* pe)		// data has invalid format
			 {
				 MessageBox::Show(pe->Message, "Error");
			 }
 
			 InternalRefFreqError = (int)(CaptFreq - 100000000);

			 if (abs(InternalRefFreqError) > 100000)
			 {
				 MessageBox::Show(String::Concat(S"Frequency error too large (greater than 100 KHz). \n\r"
				 S"Check input is close to 100.0 MHz. "), S"Out of Range",
				 MessageBoxButtons::OK, MessageBoxIcon::Error);

				 updateIntRefFreqError = false;

				 return;
			 }

			 updateIntRefFreqError = true;
			 return;
		 }
};
}

// Routine to read in reference parameters to frequency selector.
// Not part of the above object, since that object does not exist at startup
// It's in this module because it matches the reference parameter save code in OKbutton_Click

void LoadRefFile(FrequencySelector* Fs, String* StartUpPath);
